@@ -36,6 +36,7 @@ import Data.Proxy (Proxy (Proxy))
 import Data.Text (Text)
 import HsLua hiding (Div)
 import Text.Pandoc.Lua.Marshal.Attr (peekAttr, pushAttr)
+import Text.Pandoc.Lua.Marshal.Caption (peekCaptionFuzzy, pushCaption)
 import Text.Pandoc.Lua.Marshal.Content
   ( Content (..), contentTypeDescription, peekContent, pushContent
   , peekDefinitionItem )
@@ -47,8 +48,7 @@ import Text.Pandoc.Lua.Marshal.ListAttributes
   ( peekListAttributes, pushListAttributes )
 import Text.Pandoc.Lua.Marshal.Shared (walkBlocksAndInlines)
 import Text.Pandoc.Lua.Marshal.TableParts
-  ( peekCaptionFuzzy, pushCaption
-  , peekColSpec, pushColSpec
+  ( peekColSpec, pushColSpec
   , peekTableBody, pushTableBody
   , peekTableFoot, pushTableFoot
   , peekTableHead, pushTableHead
@@ -109,23 +109,44 @@ pushBlocks xs = do
   setmetatable (nth 2)
 {-# INLINABLE pushBlocks #-}
 
--- | Try extra hard to retrieve an Block value from the stack. Treats
+-- | Unmarshal a table as Block value by calling the @__toblock@ metamethod
+-- first.
+peekBlockMetamethod :: LuaError e
+                    => Peeker e Block
+peekBlockMetamethod idx = do
+  absidx <- liftLua $ absindex idx
+  liftLua (getmetafield absidx "__toblock") >>= \case
+    TypeNil      -> failPeek "object has no __toblock metamethod"
+    TypeFunction -> do
+      liftLua (pushvalue absidx)
+      liftLua (pcall 1 1 Nothing) >>= \case
+        OK   -> peekBlock top `lastly` pop 1
+        _err -> do
+          msg <- peekByteString top `lastly` pop 1
+          failPeek $ "failure in __toblock: " <> msg
+    _otherType   -> do
+      liftLua (pop 1)   -- drop "__toblock" field
+      failPeek "__toblock metafield does not contain a function"
+
+-- | Try extra hard to retrieve a Block value from the stack. Treats
 -- bare strings as @Str@ values.
 peekBlockFuzzy :: LuaError e
                => Peeker e Block
 peekBlockFuzzy idx =
        peekBlock idx
+  <|> peekBlockMetamethod idx
   <|> (Plain <$!> peekInlinesFuzzy idx)
   <|> (failPeek =<<
        typeMismatchMessage "Block or list of Inlines" idx)
 {-# INLINABLE peekBlockFuzzy #-}
 
 -- | Try extra-hard to return the value at the given index as a list of
--- inlines.
+-- 'Block's.
 peekBlocksFuzzy :: LuaError e
                 => Peeker e [Block]
 peekBlocksFuzzy idx =
-      peekList peekBlockFuzzy idx
+      ((:[]) <$> peekBlockMetamethod idx)
+  <|> peekList peekBlockFuzzy idx
   <|> (pure <$!> peekBlockFuzzy idx)
   <|> (failPeek =<<
        typeMismatchMessage "Block, list of Blocks, or compatible element" idx)
@@ -307,7 +328,7 @@ setBlockContent _ = \case
       c -> throwM . luaException @e $
            "expected definition items, got " <> contentTypeDescription c
     listItemContent = \case
-      ContentBlocks blks    -> [blks]
+      ContentBlocks blks    -> map (:[]) blks
       ContentLines lns      -> map ((:[]) . Plain) lns
       ContentListItems itms -> itms
       c -> throwM . luaException @e $
@@ -332,17 +353,20 @@ blockConstructors =
     ### liftPure BlockQuote
     <#> blocksParam
     =#> blockResult "BlockQuote element"
+    #? "Creates a block quote element"
 
   , defun "BulletList"
     ### liftPure BulletList
     <#> blockItemsParam "list items"
     =#> blockResult "BulletList element"
+    #? "Creates a bullet list."
 
   , defun "CodeBlock"
     ### liftPure2 (\code mattr -> CodeBlock (fromMaybe nullAttr mattr) code)
-    <#> textParam "text" "code block content"
+    <#> textParam "text" "code string"
     <#> optAttrParam
     =#> blockResult "CodeBlock element"
+    #? "Creates a code block element."
 
   , defun "DefinitionList"
     ### liftPure DefinitionList
@@ -353,22 +377,25 @@ blockConstructors =
                   "{{Inlines, {Blocks,...}},...}"
                   "content" "definition items"
     =#> blockResult "DefinitionList element"
+    #? "Creates a definition list, containing terms and their explanation."
 
   , defun "Div"
     ### liftPure2 (\content mattr -> Div (fromMaybe nullAttr mattr) content)
     <#> blocksParam
     <#> optAttrParam
     =#> blockResult "Div element"
+    #? "Creates a div element"
 
   , defun "Figure"
     ### liftPure3 (\content mcapt mattr ->
                      let attr = fromMaybe nullAttr mattr
                          capt = fromMaybe (Caption mempty mempty) mcapt
                      in Figure attr capt content)
-    <#> parameter peekBlocksFuzzy "Blocks" "content" "figure content"
+    <#> parameter peekBlocksFuzzy "Blocks" "content" "figure block content"
     <#> opt (parameter peekCaptionFuzzy "Caption" "caption" "figure caption")
     <#> optAttrParam
-    =#> blockResult "Figure element"
+    =#> blockResult "Figure object"
+    #? "Creates a [[Figure]] element."
 
   , defun "Header"
     ### liftPure3 (\lvl content mattr ->
@@ -377,40 +404,47 @@ blockConstructors =
     <#> parameter peekInlinesFuzzy "Inlines" "content" "inline content"
     <#> optAttrParam
     =#> blockResult "Header element"
+    #? "Creates a header element."
 
   , defun "HorizontalRule"
     ### return HorizontalRule
     =#> blockResult "HorizontalRule element"
+    #? "Creates a horizontal rule."
 
   , defun "LineBlock"
     ### liftPure LineBlock
     <#> parameter (peekList peekInlinesFuzzy) "{Inlines,...}" "content" "lines"
     =#> blockResult "LineBlock element"
+    #? "Creates a line block element."
 
   , defun "OrderedList"
     ### liftPure2 (\items mListAttrib ->
                      let defListAttrib = (1, DefaultStyle, DefaultDelim)
                      in OrderedList (fromMaybe defListAttrib mListAttrib) items)
-    <#> blockItemsParam "ordered list items"
+    <#> blockItemsParam "list items"
     <#> opt (parameter peekListAttributes "ListAttributes" "listAttributes"
-                       "specifier for the list's numbering")
+                       "list parameters")
     =#> blockResult "OrderedList element"
+    #? "Creates an ordered list."
 
   , defun "Para"
     ### liftPure Para
-    <#> parameter peekInlinesFuzzy "Inlines" "content" "paragraph content"
+    <#> parameter peekInlinesFuzzy "Inlines" "content" "inline content"
     =#> blockResult "Para element"
+    #? "Creates a para element."
 
   , defun "Plain"
     ### liftPure Plain
-    <#> parameter peekInlinesFuzzy "Inlines" "content" "paragraph content"
+    <#> parameter peekInlinesFuzzy "Inlines" "content" "inline content"
     =#> blockResult "Plain element"
+    #? "Creates a plain element."
 
   , defun "RawBlock"
     ### liftPure2 RawBlock
-    <#> parameter peekFormat "Format" "format" "format of content"
+    <#> parameter peekFormat "string" "format" "format of content"
     <#> textParam "text" "raw content"
     =#> blockResult "RawBlock element"
+    #? "Creates a raw content block of the specified format."
 
   , defun "Table"
     ### (\capt colspecs thead tbodies tfoot mattr ->
@@ -426,23 +460,27 @@ blockConstructors =
     <#> parameter peekTableFoot "TableFoot" "foot" "table foot"
     <#> optAttrParam
     =#> blockResult "Table element"
+    #? "Creates a table element."
   ]
  where
   blockResult = functionResult pushBlock "Block"
   blocksParam = parameter peekBlocksFuzzy "Blocks" "content" "block content"
-  blockItemsParam = parameter peekItemsFuzzy "List of Blocks" "content"
+  blockItemsParam = parameter peekItemsFuzzy "{Blocks,...}" "items"
   peekItemsFuzzy idx = peekList peekBlocksFuzzy idx
     <|> ((:[]) <$!> peekBlocksFuzzy idx)
 
-  optAttrParam = opt (parameter peekAttr "Attr" "attr" "additional attributes")
+  optAttrParam = opt (parameter peekAttr "Attr" "attr" "element attributes")
 
 
 -- | Constructor for a list of `Block` values.
 mkBlocks :: LuaError e => DocumentedFunction e
 mkBlocks = defun "Blocks"
   ### liftPure id
-  <#> parameter peekBlocksFuzzy "Blocks" "blocks" "block elements"
+  <#> parameter peekBlocksFuzzy "Blocks" "block_like_elements"
+      ("List where each element can be treated as a [[Block]] value, " <>
+       "or a single such value.")
   =#> functionResult pushBlocks "Blocks" "list of block elements"
+  #? "Creates a [[Blocks]] list."
 
 --
 -- walk
