@@ -32,7 +32,7 @@ peekTableHead = peekUD typeTableHead
 
 -- | Row object type.
 typeTableHead :: LuaError e => DocumentedType e TableHead
-typeTableHead = deftype "pandoc TableHead"
+typeTableHead = deftype "TableHead"
   [ operation Eq $ defun "__eq"
      ### liftPure2 (\a b -> fromMaybe False ((==) <$> a <*> b))
      <#> parameter (optional . peekTableHead) "TableHead" "self" ""
@@ -72,6 +72,8 @@ mkTableHead = defun "TableHead"
   ### liftPure2 (\mRows mAttr -> TableHead
                   (fromMaybe nullAttr mAttr)
                   (fromMaybe [] mRows))
-  <#> opt (parameter (peekList peekRowFuzzy) "{Row,...}" "rows" "header rows")
+  <#> opt (parameter (peekList peekRowFuzzy) "{Row,...}" "rows"
+           "list of table rows")
   <#> opt (parameter peekAttr "Attr" "attr" "table head attributes")
   =#> functionResult pushTableHead "TableHead" "new TableHead object"
+  #? "Creates a table head."
