@@ -1,7 +1,7 @@
 {-# LANGUAGE LambdaCase           #-}
 {-# LANGUAGE OverloadedStrings    #-}
 {- |
-Copyright               : © 2021-2022 Albert Krewinkel
+Copyright               : © 2021-2024 Albert Krewinkel
 SPDX-License-Identifier : MIT
 Maintainer              : Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
 
@@ -9,9 +9,7 @@ Marshaling/unmarshaling functions of types that are used exclusively
 with tables.
 -}
 module Text.Pandoc.Lua.Marshal.TableParts
-  ( peekCaption
-  , pushCaption
-  , peekColSpec
+  ( peekColSpec
   , pushColSpec
   , peekRow
   , peekRowFuzzy
@@ -33,30 +31,12 @@ import Control.Monad ((<$!>))
 import HsLua
 import Text.Pandoc.Lua.Marshal.Alignment (peekAlignment, pushAlignment)
 import Text.Pandoc.Lua.Marshal.Attr (peekAttr, pushAttr)
-import {-# SOURCE #-} Text.Pandoc.Lua.Marshal.Block
-  ( peekBlocksFuzzy, pushBlocks )
-import {-# SOURCE #-} Text.Pandoc.Lua.Marshal.Inline
-  ( peekInlinesFuzzy, pushInlines )
 import Text.Pandoc.Lua.Marshal.List (pushPandocList)
 import Text.Pandoc.Lua.Marshal.Row
 import Text.Pandoc.Lua.Marshal.TableFoot
 import Text.Pandoc.Lua.Marshal.TableHead
 import Text.Pandoc.Definition
 
--- | Push Caption element
-pushCaption :: LuaError e => Caption -> LuaE e ()
-pushCaption (Caption shortCaption longCaption) = do
-  newtable
-  addField "short" (maybe pushnil pushInlines shortCaption)
-  addField "long" (pushBlocks longCaption)
-
--- | Peek Caption element
-peekCaption :: LuaError e => Peeker e Caption
-peekCaption = retrieving "Caption" . \idx -> do
-  short <- optional $ peekFieldRaw peekInlinesFuzzy "short" idx
-  long <- peekFieldRaw peekBlocksFuzzy "long" idx
-  return $! Caption short long
-
 -- | Push a ColSpec value as a pair of Alignment and ColWidth.
 pushColSpec :: LuaError e => Pusher e ColSpec
 pushColSpec = pushPair pushAlignment pushColWidth
